==> guessnet_0.28-1.tar.gz: guessnet-0.28-1/src/PeerScanner.cc <==
/*
 * Scan for the existance of a specific peer using fake ARP requests
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 * Originally based on laptop-netconf.c by Matt Kern <matt@debian.org>
 * Which in turn was nased on divine.c by Felix von Leitner <felix@fefe.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "PeerScanner.h"
#include "PacketMaker.h"
#include "Environment.h"

#include <netinet/in.h> // ntohs, htons, ...

/*
extern "C" {
#include <libnet.h>
}
*/

using namespace std;


void PeerScanner::handleARP(struct libnet_arp_hdr* arp_header) throw ()
{
	MutexLock lock(candMutex);

	// Parse and check the arp header
	if (ntohs (arp_header->ar_op) == ARPOP_REPLY)
	{
		//in_addr* ipv4_him = arp_get_tip(arp_header);
		in_addr* ipv4_him = arp_get_sip(arp_header);
		ether_addr* mac_him = arp_get_sha(arp_header);

		debug("Got ARP reply from %.*s %.*s", PFSTR(fmt(IPAddress(*ipv4_him))), PFSTR(fmt(*mac_him)));

		//IPv4_FROM_LIBNET(ipv4_me, arp_header->ar_tpa);
		//IPv4_FROM_ARP(ipv4_him, arp_header->ar_spa);

		for (list<const PeerScan*>::const_iterator i = candidates.begin();
				i != candidates.end(); i++)
		{
			// Check if IP and MAC addresses match
			if (IPv4_MATCHES(&(*i)->ip(), ipv4_him))
			{
				if (MAC_MATCHES(&(*i)->mac(), mac_him))
				{
					debug("ARP reply from %.*s %.*s matches", PFSTR(fmt(IPAddress(*ipv4_him))), PFSTR(fmt(*mac_him)));
					found(*i);
				}
				else
				{
					// If only the IP matches, check if the scan is an IP-only
					// scan
					struct ether_addr zeroAddr;
					bzero(&zeroAddr, sizeof(struct ether_addr));
					if (MAC_MATCHES(&(*i)->mac(), &zeroAddr))
					{
						debug("ARP reply from %.*s %.*s matches", PFSTR(fmt(IPAddress(*ipv4_him))), PFSTR(fmt(*mac_him)));
						found(*i);
					}
				}
			}
		}
	}
}


void PeerScanner::addCandidate(const PeerScan* scan)
	throw ()
{
	{
		MutexLock lock(candMutex);
		candidates.push_back(scan);
	}

	// Build and send the arp probe
	PacketMaker pm(sender);
	Buffer pkt = pm.makeARPRequest(scan->ip());

	verbose("Sending 10 ARP probes, 1 every second...\n");

	// Enqueue the packet for sending
	sender.post(pkt, 1000, 10000);
}

// vim:set ts=4 sw=4:

==> guessnet_0.33.orig.tar.gz: guessnet-0.33/src/PeerScanner.cc <==
/*
 * Scan for the existance of a specific peer using fake ARP requests
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 * Originally based on laptop-netconf.c by Matt Kern <matt@debian.org>
 * Which in turn was nased on divine.c by Felix von Leitner <felix@fefe.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "PeerScanner.h"
#include "PacketMaker.h"
#include "Environment.h"

#include <netinet/in.h> // ntohs, htons, ...

/*
extern "C" {
#include <libnet.h>
}
*/

using namespace std;


void PeerScanner::handleARP(const NetBuffer& arp) throw ()
{
	MutexLock lock(candMutex);

	// Parse and check the arp header
	const libnet_arp_hdr* arp_header = arp.cast<libnet_arp_hdr>();
	if (ntohs (arp_header->ar_op) == ARPOP_REPLY)
	{
		//in_addr* ipv4_him = arp_get_tip(arp_header);
		in_addr* ipv4_him = arp_get_sip(arp_header);
		ether_addr* mac_him = arp_get_sha(arp_header);

		debug("Got ARP reply from %.*s %.*s\n", PFSTR(fmt(IPAddress(*ipv4_him))), PFSTR(fmt(*mac_him)));

		//IPv4_FROM_LIBNET(ipv4_me, arp_header->ar_tpa);
		//IPv4_FROM_ARP(ipv4_him, arp_header->ar_spa);

		for (list<const PeerScan*>::const_iterator i = candidates.begin();
				i != candidates.end(); i++)
		{
			// Check if IP and MAC addresses match
			if (IPv4_MATCHES(&(*i)->ip(), ipv4_him))
			{
				if (MAC_MATCHES(&(*i)->mac(), mac_him))
				{
					debug("ARP reply from %.*s %.*s matches\n", PFSTR(fmt(IPAddress(*ipv4_him))), PFSTR(fmt(*mac_him)));
					found(*i);
				}
				else
				{
					// If only the IP matches, check if the test is IP-only
					struct ether_addr zeroAddr;
					bzero(&zeroAddr, sizeof(struct ether_addr));
					if (MAC_MATCHES(&(*i)->mac(), &zeroAddr))
					{
						debug("ARP reply from %.*s %.*s matches\n", PFSTR(fmt(IPAddress(*ipv4_him))), PFSTR(fmt(*mac_him)));
						found(*i);
					}
				}
			}
		}
	}
}


void PeerScanner::addCandidate(const PeerScan* scan)
	throw ()
{
	{
		MutexLock lock(candMutex);
		candidates.push_back(scan);
	}

	// Build and send the arp probe
	PacketMaker pm(sender);
	Buffer pkt = pm.makeARPRequest(scan->ip(), scan->source());

	verbose("Sending 10 ARP probes, 1 every second...\n");

	// Enqueue the packet for sending
	sender.post(pkt, 1000, 10000);
}

// vim:set ts=4 sw=4:
